part of '../flutterx_live_data.dart';

class MutableLiveData<T> extends LiveData<T> {
  MutableLiveData({T? initialValue}) : assert(initialValue is T, 'initialValue must be of type $T, got $initialValue') {
    setData(initialValue as T);
  }

  MutableLiveData.late();

  set value(T value) => setData(value);

  void postValue(FutureOr<T> value) => postData(value);
}

class MutableMediatorLiveData<T> extends MutableLiveData<T> with MediatorMixin<T> {
  MutableMediatorLiveData({T? initialValue}) : super(initialValue: initialValue);

  MutableMediatorLiveData.late() : super.late();
}

class HistoryMutableLiveData<T> extends MutableLiveData<T> with HistoryMixin<T> {
  HistoryMutableLiveData({T? initialValue}) : super(initialValue: initialValue);

  HistoryMutableLiveData.late() : super.late();
}

class StorageMutableLiveData<T> extends MutableLiveData<T> with StorageMixin<T> {
  @override
  final String key;

  @override
  final T defaultValue;

  @override
  final Map<String, String> _storage;

  @override
  final String Function(T value) _encode;

  StorageMutableLiveData({
    required this.key,
    required Map<String, String> storage,
    T? defaultValue,
    String Function(T value) encode = _defaultEncode,
    T Function(String source)? decode,
  })  : assert(defaultValue is T, 'defaultValue must be of type $T, got $defaultValue'),
        _storage = storage,
        _encode = encode,
        defaultValue = defaultValue as T,
        super(initialValue: StorageMixin.resolve(key, storage, defaultValue, decode ?? _defaultDecode));

  StorageMutableLiveData.enumeration({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    required Iterable<T> values,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: StorageMixin.encodeEnum,
            decode: (name) => StorageMixin.decodeEnum(name, values));

  StorageMutableLiveData.json({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(dynamic json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));

  StorageMutableLiveData.jsonObject({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
    dynamic Function(T value)? toJson,
    required T Function(Map<String, dynamic> json) fromJson,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            encode: (value) => jsonEncode(toJson == null ? value : toJson(value)),
            decode: (source) => fromJson(jsonDecode(source)));

  StorageMutableLiveData.int({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : int.parse(source) as T);

  StorageMutableLiveData.double({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : double.parse(source) as T);

  StorageMutableLiveData.bool({
    required String key,
    required Map<String, String> storage,
    T? defaultValue,
  }) : this(
            key: key,
            storage: storage,
            defaultValue: defaultValue,
            decode: (source) => source == 'null' ? null as T : ('true' == source) as T);

  static String _defaultEncode(dynamic value) => value.toString();

  static T _defaultDecode<T>(String source) {
    assert(source is T,
        'Default decode only supports ${source.runtimeType}, got $T. Use a custom decode function instead');
    return source as T;
  }
}
